#ifndef STEPPER_MOTOR_H
#define STEPPER_MOTOR_H


#define SYSCLK_FREQUENCY 72000000  // 主时钟频率（72 MHz）

#define STEPMOTOR_ANGLEPERSTEP  1.8  //步进电机步距角
#define STEPMOTOR_FREDIV  32         //步进电机分频系数（细分数）

#define STEPMOTOR_PERDISTANCE 2        //步进电机一转走2mm
#define STEPMOTOR_PULSEPERROUND  ((360/STEPMOTOR_ANGLEPERSTEP)*STEPMOTOR_FREDIV)   //步进电机每转所需要脉冲的脉冲个数
#define STEPMOTOR_PULSEPERDISTANCE  STEPMOTOR_PULSEPERROUND/STEPMOTOR_PERDISTANCE  //步进电机每1mm所需要脉冲的脉冲个数
#define STEPMOTOR_FREQ_ROUNDPERMIN (9375/(STEPMOTOR_FREDIV*2))   //步进电机1r/min需要配置的定时器重载值((72000000*60/STEPMOTOR_PULSEPERROUND)/72)（假设时钟分频为8）   定时时间=60s/(每转需要的脉冲个数)  秒冲个数*2是因为只认高-低
#define STEPMOTOR_MAX_SPEED(psc, arr)  (SYSCLK_FREQUENCY / ((psc + 1) * (arr + 1)) * 60 / STEPMOTOR_PULSEPERROUND)


typedef enum
{
    SPEED_NONE,   // 无速度变化
    SPEED_ACC,    // 加速
    SPEED_DEC,    // 减速
    SPEED_STOP,   // 停止
} SpeedStatus;


// 定义步进电机配置结构体
typedef struct {
    unsigned stepcount;       // 步数（脉冲数）
    unsigned direction;       // 方向 (0: 逆时针, 1: 顺时针)
    unsigned curSpeed;        // 当前速度 (步/秒)
    unsigned desSpeed;        // 目标速度 (步/秒)
	  unsigned maxSpeed;        //最大转速
    SpeedStatus speedStatus;  // 速度状态
    unsigned stepcountMode;   // 脉冲计数模式
    unsigned curlocation;     // 当前位置
    unsigned deslocation;     // 目标位置
    unsigned acceleration;    // 加速度 (步/秒^2)
    unsigned deceleration;    // 减速度 (步/秒^2)
	  uint16_t  psc;            //预分频器 
    uint16_t  arr;               //自动重装载寄存器 
} StepMotor;


// 更新电机状态
void Update_Motor_Status(StepMotor *motor, SpeedStatus status, uint32_t targetSpeed, uint32_t acceleration);
;

void Set_Direction(int direction);

// 使能或禁用步进电机
void Enable_Stepper(bool enable);

// 设置PWM频率
void Set_PWM_Frequency(uint32_t frequency)；

// 梯形加减速算法
void StepMotor(int steps, int direction, int max_speed, int acceleration);

void Delay(uint32_t delay);

void Timer_Frequency(int & frequency,StepMotor *stepMotor);

void TIM3_Init(uint16_t &psc,uint16_t &arr);



#endif // STEPPER_MOTOR_H