#include "uart_log.h"
#include <stdarg.h>
#include <string.h>
#include <stdio.h>  // 包含<stdio.h>头文件

// 缓冲区大小
#define SERIAL_BUFFER_SIZE 128

// USART1 GPIO初始化
void USART1_GPIO_Init(void) {
    // 使能GPIOA和USART1时钟
    RCC->APB2ENR |= RCC_APB2ENR_IOPAEN | RCC_APB2ENR_USART1EN;
    GPIOA->CRH &= ~(0xF << 20);
    GPIOA->CRH |= (0xB << 20);  

    GPIOA->CRH &= ~(0xF << 24);  // 清除模式控制位
    GPIOA->CRH |= (0x4 << 24);   // 设置为浮空输入
}

// USART1初始化
void USART1_Init(void) {
    // 设置波特率
    uint32_t BRR = (uint32_t)(SystemCoreClock / 16 / 115200);
    USART1->BRR = BRR;

    // 配置USART1
    USART1->CR1 = USART_CR1_UE | USART_CR1_TE | USART_CR1_RE; // 使能USART1、发送器和接收器
    USART1->CR2 = 0; // 不使用流控制
    USART1->CR3 = 0; // 不使用额外功能
}

// 发送一个字符
void USART1_SendChar(char ch) {
    while (!(USART1->SR & USART_SR_TXE)); // 等待发送缓冲区为空
    USART1->DR = (ch & 0xFF); // 发送字符
}

// 发送字符串
void USART1_SendString(const char *str) {
    while (*str) {
        USART1_SendChar(*str++);
    }
}

// 格式化字符串并发送
void USART1_SendFormatted(const char *format, ...) {
    char buffer[SERIAL_BUFFER_SIZE];
    va_list args;
    va_start(args, format);
    vsnprintf(buffer, sizeof(buffer), format, args);
    va_end(args);
    USART1_SendString(buffer);
}

// 初始化串口日志
void Serial_Init(void) {
    USART1_GPIO_Init();
    USART1_Init();
}

// 打印字符串
void Serial_Print(const char *str) {
    USART1_SendString(str);
}

// 打印字符
void Serial_Write(char ch) {
    USART1_SendChar(ch);
}

// 打印格式化字符串
void Serial_Printf(const char *format, ...) {
    va_list args;
    va_start(args, format);
    USART1_SendFormatted(format, args);
    va_end(args);
}


